import java.util.HashSet;

class GraphAdjList implements IGraph{
	int[] neighborIndex;
	int[] neighbors;
	public void setGraph(int[] vertices, Edge[] edges){
		neighborIndex = new int[vertices.length];
		if (edges == null)
			neighbors = null;
		else{
			neighbors = new int[edges.length*2];
			int curIndex = 0;
			for(int v : vertices){
				neighborIndex[v] = curIndex; // Isolierte am Ende bekommen einen Index auerhalb von neighbors
				for(Edge e : edges){
					if (e.v1 == v)
						neighbors[curIndex++] = e.v2;
					if (e.v2 == v)
						neighbors[curIndex++] = e.v1;
				}
			}
		}
	}
	public int[] getVertices(){
		int[] res = new int[neighborIndex.length];
		for(int i = 0; i < res.length; i++)
			res[i] = i;
		return res;
	}
	public Edge[] getEdges(){
		if (neighbors == null) return null;
		HashSet<Edge> res = new HashSet<Edge>();
		for(int i = 0; i < neighborIndex.length; i++){
			int until = (i < (neighborIndex.length-1)) ? neighborIndex[i+1]-1 : neighbors.length-1;
			for(int j = neighborIndex[i]; j <= until; j++){
				Edge e = new Edge(i,neighbors[j]);
				if (!res.contains(e))
					res.add(e);
			}
		}
		if (res.size() > 0)
			return res.toArray(new Edge[res.size()]);
		return null;
	}
	public boolean isAdjacent(int v1, int v2){
		if (neighbors == null) return false;
		int until = (v1 < (neighborIndex.length-1)) ? neighborIndex[v1+1]-1 : neighbors.length-1;
		for(int j = neighborIndex[v1]; j <= until; j++){
			if (neighbors[j] == v2)
				return true;
		}
		return false;
	}
	public int getDegree(int vertex){
		if (neighbors == null) return 0;
		int until = (vertex < (neighborIndex.length-1)) ? neighborIndex[vertex+1]-1 : neighbors.length-1;
		return until - neighborIndex[vertex]+1;
	}
	public int getSize(){
		return neighborIndex.length*4 + (neighbors == null ? 0 : neighbors.length*4);
	}
}