import java.util.HashSet;
import java.util.Random;

public class GraphChecker{
	public static void main(String[] args){
		GraphAdjMat gam = new GraphAdjMat();
		GraphAdjList gal = new GraphAdjList();
		for(int n = 250; n <= 5000; n+=250){
			Check(gam,n,n*(n-1)/2 / 50);
		}
		for(int n = 250; n <= 5000; n+=250){
			Check(gal,n,n*(n-1)/2 / 50);
		}
	}
	static int COUNT_LOOP = 100; // Number of random input parameters are generated
	static int TIME_FACTOR = 100; // Number of times a certain function call is repeated
	static int COUNT_SAMPLES = 1; // Number of different graphs with the same vertex/edge count (only 1 with max edge count)
	
	private static Random r = new Random();
	static boolean Check(IGraph graph, int cntVertex, int cntEdge){
		try{
			long minDeg = Long.MAX_VALUE;
			long maxDeg = 0;
			long sumDeg = 0;
			long minAdj = Long.MAX_VALUE;
			long maxAdj = 0;
			long sumAdj = 0;
			for(int s = 0; s < COUNT_SAMPLES; s++){
				if (!GraphGenerator.generate(graph,cntVertex,cntEdge))
					System.out.println(graph.getClass().getSimpleName() + " generation failed!");
				else{
					for(int i = 0; i < COUNT_LOOP; i++){
						// Measuring getDegree
						int v = r.nextInt(cntVertex);
						long start = System.nanoTime();
						for(int t = 0; t < TIME_FACTOR; t++)
							graph.getDegree(v);
						long end = System.nanoTime();
						long len = end - start;
						minDeg = Math.min(minDeg,len);
						maxDeg = Math.max(maxDeg,len);
						sumDeg += len;

						// Measuring isAdjacent
						int i1 = r.nextInt(cntVertex);
						int i2 = r.nextInt(cntVertex);
						start = System.nanoTime();
						for(int t = 0; t < TIME_FACTOR; t++)
							graph.isAdjacent(i1,i2);
						end = System.nanoTime();
						len = end - start;
						minAdj = Math.min(minAdj,len);
						maxAdj = Math.max(maxAdj,len);
						sumAdj += len;
					}
				}
			}
			System.out.println(graph.getClass().getSimpleName() + "\t" + cntVertex+"\t"+cntEdge+"\t"+graph.getSize()+"\t"+minDeg+"\t"+maxDeg+"\t"+(sumDeg/(COUNT_SAMPLES*COUNT_LOOP))+"\t"+minAdj+"\t"+maxAdj+"\t"+(sumAdj/(COUNT_SAMPLES*COUNT_LOOP)));
		}
		catch(Exception ex){
			ex.printStackTrace();
			return false;
		}
		return true;
	}
}


